import tatsumaki.wrapper
import tatsumaki.data_structures
import tatsumaki.error

wrapper = tatsumaki.wrapper
data_structures = tatsumaki.data_structures
error = tatsumaki.error

__all__ = [
    'wrapper',
]

name = "tatsumaki.py"
version = "0.3"



import aiohttp
from ratelimit import limits
from typing import Union

import tatsumaki.data_structures as ds


class ApiWrapper:
    def __init__(self, key):
        self.key = key
        self.base_url = "https://api.tatsu.gg/v1/"
        self.headers = {"Authorization": key}

    @limits(calls=60, period=60)
    async def request(self, url):
        """Directly interact with the API to get the unfiltered results."""
        async with aiohttp.ClientSession() as session:
            async with session.get(url=self.base_url + url, headers=self.headers) as result:
                if result.status != 200:
                    return result.raise_for_status()
                return await result.json()

    async def get_profile(self, user_id: int) -> Union[Exception, ds.UserProfile]:
        """Gets a user's profile. Returns a user object on success."""
        try:
            result = await self.request(f"users/{user_id}/profile")
        except Exception as e:
            return e
        user = ds.UserProfile(
            avatar_url=result.get('avatar_url', None),
            credits_=result.get('credits', None),
            discriminator=result.get('discriminator', None),
            user_id=result.get('id', None),
            info_box=result.get('info_box', None),
            reputation=result.get('reputation', None),
            title=result.get('title', None),
            tokens=result.get('tokens', None),
            username=result.get('username', None),
            xp=result.get('xp', None),
            original=result
        )
        return user

    async def get_member_ranking(self, guild_id: int, user_id: int) -> Union[ds.RankingObject, Exception]:
        """Gets the all-time ranking for a guild member. Returns a guild member ranking object on success."""
        try:
            result = await self.request(f"/guilds/{guild_id}/rankings/members/{user_id}/all")
        except Exception as e:
            return e
        rank = self.ranking_object(result)
        return rank

    @staticmethod
    def ranking_object(result) -> ds.RankingObject:
        """Initiate the rank profile"""
        rank = ds.RankingObject(
            guild_id=result.get('guild_id', None),
            rank=result.get('rank', None),
            score=result.get('score', None),
            user_id=result.get('user_id', None),
            original=result
        )
        return rank

    async def get_guild_rankings(self, guild_id, offset=0) -> Union[ds.GuildRankings, Exception]:
        """Gets all-time rankings for a guild. Returns a guild rankings object on success."""
        try:
            result = await self.request(f"/guilds/{guild_id}/rankings/all?offset={offset}")
        except Exception as e:
            return e
        rankings = ds.GuildRankings(
            guild_id=result.get('guild_id', None),
            rankings=[self.ranking_object(i) for i in result.get('rankings', [{}])],
            original=result
        )
        return rankings



class UserProfile:
    def __init__(self, avatar_url, credits_, discriminator, user_id, info_box,
                 reputation, title, tokens, username, xp, original):
        self.avatar_url: str = avatar_url
        self.credits: int = credits_
        self.discriminator: str = discriminator
        self.user_id: int = int(user_id) if user_id else user_id
        self.info_box: str = info_box
        self.reputation: int = reputation
        self.title: str = title
        self.tokens: int = tokens
        self.username: str = username
        self.xp: int = xp
        self.original: dict = original


class GuildRankings:
    def __init__(self, guild_id, rankings, original):
        self.guild_id: int = int(guild_id) if guild_id else guild_id
        self.rankings: [RankingObject] = rankings
        self.original: dict = original


class RankingObject:
    def __init__(self, rank, score, user_id, original, guild_id=None):
        self.rank: int = rank
        self.score: int = score
        self.user_id: int = int(user_id) if user_id else user_id
        self.original: dict = original
        self.guild_id: int = int(guild_id) if guild_id else guild_id



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

setup(
    name='tatsu.py',
    version='1.0',
    description="An async python API Wrapper for the Tatsu API.",
    install_requires=[
        'aiohttp==3.7.3',
        'ratelimit'
    ],
    license='MIT License',
    long_description=(here / 'README.md').read_text(encoding='utf-8'),
    long_description_content_type='text/markdown',
    url="https://github.com/PumPum7/Tatsu.py",
    author="Pum",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    keywords="api, tatsu, api wrapper",
    packages=["tatsumaki"],
    project_urls={
        "Tatsu": "https://tatsu.gg/",
        "Source": "https://github.com/PumPum7/Tatsu.py"
    }
)
